var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/**
 * Represents a ContainerExecution class that handles the execution of a launchable handler.
 * @template Input - The type of input expected by the launchable handler.
 */
export default class ContainerExecution {
    /**
     * Constructor for creating an instance of a class with a LaunchableHandlerType input.
     * @param {LaunchableHandlerType<Input>} handler - The handler for the launchable type.
     * @returns None
     */
    constructor(handler) {
        this.handler = handler;
    }
    /**
     * Executes the function asynchronously, invoking the specified inputs and handling the response accordingly.
     * If the response code is 200, the process exits with code 0; otherwise, it exits with code 1.
     * @returns None
     */
    execute() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                // Invoke
                const resp = yield this.invoke(this.getInputs());
                // Return
                console.debug(resp);
                if (!resp.err &&
                    resp.data &&
                    (resp.data.statusCode == 200 || (resp.data.getCode && resp.data.getCode() == 200)))
                    process.exit(0);
                else
                    process.exit(1);
            }
            catch (e) {
                console.error(e);
                process.exit(1);
            }
        });
    }
    /**
     * Retrieves inputs from the process arguments and parses them as JSON.
     * @returns {any} The parsed input object from process arguments.
     */
    getInputs() {
        let input = {};
        console.debug(process.argv);
        try {
            input = JSON.parse(process.argv[2]);
        }
        catch (e) {
            console.log('[IKApi Launchable Runtime] - Error while decoding process arguments! (can be desconsider if no inputs were provided)', e.message);
        }
        return input;
    }
    /**
     * Builds a context object for AWS Lambda function.
     * @param {CallableFunction} callback - The callback function to be executed.
     * @returns {Context} A context object with AWS Lambda properties.
     */
    buildContext(callback) {
        return {
            awsRequestId: `${Date.now()}-${Date.now()}`,
            callbackWaitsForEmptyEventLoop: true,
            getRemainingTimeInMillis: () => 0,
            done: (err, data) => callback(err, data),
            fail: err => callback(err),
            succeed: res => callback(null, res),
        };
    }
    /**
     * Asynchronously invokes a function with the given input and returns a promise that resolves
     * with an object containing an error message or data response.
     * @param {any} input - The input to be passed to the function.
     * @returns {Promise<{ err?: string; data?: any }>} A promise that resolves with an object
     * containing an error message or data response (serialized).
     */
    invoke(input) {
        return __awaiter(this, void 0, void 0, function* () {
            // eslint-disable-next-line no-async-promise-executor
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    const context = this.buildContext((err, data) => {
                        resolve({ err, data });
                    });
                    const resp = yield this.handler(input, context);
                    if (resp)
                        resolve({ data: resp });
                }
                catch (e) {
                    reject(e);
                }
            }));
        });
    }
}
//# sourceMappingURL=ContainerExec.js.map