import { Context } from 'aws-lambda'

import { LaunchableHandlerType } from './types.js'

/**
 * Represents a ContainerExecution class that handles the execution of a launchable handler.
 * @template Input - The type of input expected by the launchable handler.
 */
export default class ContainerExecution<Input> {
  /**
   * A private readonly property that holds a handler of LaunchableHandlerType with a given Input type.
   */
  private readonly handler: LaunchableHandlerType<Input>

  /**
   * Constructor for creating an instance of a class with a LaunchableHandlerType input.
   * @param {LaunchableHandlerType<Input>} handler - The handler for the launchable type.
   * @returns None
   */
  constructor(handler: LaunchableHandlerType<Input>) {
    this.handler = handler
  }

  /**
   * Executes the function asynchronously, invoking the specified inputs and handling the response accordingly.
   * If the response code is 200, the process exits with code 0; otherwise, it exits with code 1.
   * @returns None
   */
  public async execute() {
    try {
      // Invoke
      const resp = await this.invoke(this.getInputs())
      // Return
      console.debug(resp)
      if (
        !resp.err &&
        resp.data &&
        (resp.data.statusCode == 200 || (resp.data.getCode && resp.data.getCode() == 200))
      )
        process.exit(0)
      else process.exit(1)
    } catch (e) {
      console.error(e)
      process.exit(1)
    }
  }

  /**
   * Retrieves inputs from the process arguments and parses them as JSON.
   * @returns {any} The parsed input object from process arguments.
   */
  private getInputs() {
    let input: any = {}
    console.debug(process.argv)
    try {
      input = JSON.parse(process.argv[2])
    } catch (e) {
      console.log(
        '[IKApi Launchable Runtime] - Error while decoding process arguments! (can be desconsider if no inputs were provided)',
        e.message
      )
    }
    return input
  }

  /**
   * Builds a context object for AWS Lambda function.
   * @param {CallableFunction} callback - The callback function to be executed.
   * @returns {Context} A context object with AWS Lambda properties.
   */
  private buildContext(callback: CallableFunction): Context {
    return {
      awsRequestId: `${Date.now()}-${Date.now()}`,
      callbackWaitsForEmptyEventLoop: true,
      getRemainingTimeInMillis: () => 0,
      done: (err, data) => callback(err, data),
      fail: err => callback(err),
      succeed: res => callback(null, res),
    } as any as Context
  }

  /**
   * Asynchronously invokes a function with the given input and returns a promise that resolves
   * with an object containing an error message or data response.
   * @param {any} input - The input to be passed to the function.
   * @returns {Promise<{ err?: string; data?: any }>} A promise that resolves with an object
   * containing an error message or data response (serialized).
   */
  private async invoke(input: any) {
    // eslint-disable-next-line no-async-promise-executor
    return new Promise<{ err?: string; data?: any }>(async (resolve, reject) => {
      try {
        const context = this.buildContext((err, data) => {
          resolve({ err, data })
        })
        const resp = await this.handler(input, context)
        if (resp) resolve({ data: resp })
      } catch (e) {
        reject(e)
      }
    })
  }
}
