import ContainerExecution from './ContainerExec.js'
import { LaunchableHandlerType } from './types.js'
import Utils from '../../Util/Utils.js'

/**
 * Represents a Launchable class that can be used to handle launchable items.
 * @template Input - The type of input expected by the LaunchableHandlerType.
 */
export default class Launchable<Input = any> {
  /**
   * A private readonly property that holds a handler for launching with a specific input type.
   * @type {LaunchableHandlerType<Input>}
   */
  private readonly handler: LaunchableHandlerType<Input>

  /**
   * Constructor for creating an instance of a class with a LaunchableHandlerType input.
   * @param {LaunchableHandlerType<Input>} handler - The handler for the launchable type.
   * @returns None
   */
  constructor(handler: LaunchableHandlerType<Input>) {
    this.handler = handler
  }

  /**
   * Returns the export function based on whether the object is a container or not.
   * If it is a container, it returns the container export function, otherwise it returns the handler function.
   * @returns {CallableFunction} The export function to be used.
   */
  public getExport(): CallableFunction {
    return this.isContainer() ? this.getContainerExport() : this.handler
  }

  /**
   * Checks if the current element is a container by calling the isHybridlessContainer method from the Utils class.
   * @returns {boolean} - true if the element is a container, false otherwise.
   */
  private isContainer(): boolean {
    return Utils.isHybridlessContainer()
  }

  /**
   * Returns a callable function that executes a ContainerExecution instance and returns a promise.
   * @returns {CallableFunction} A function that when called, executes a ContainerExecution instance.
   */
  private getContainerExport(): CallableFunction {
    // emulate lambda invocation
    const ce = new ContainerExecution(this.handler)
    ce.execute().then(() => {})
    // return empty function, we are all done
    return () => {}
  }
}
